<?php

/**
 * @file
 * Contains \Drupal\cg_payment\Controller\PaymentCompletePage.
 */

namespace Drupal\cg_payment\Controller;

use Creditguard\CgRedirectReturn;
use Drupal\cg_payment\TransactionInterface;
use Drupal\cg_payment\Utility\TransactionTrait;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Exception\AccessDeniedHttpException;
use Symfony\Component\HttpKernel\Exception\NotFoundHttpException;

/**
 * Class PaymentCompletePage.
 *
 * @package Drupal\cg_payment\Controller
 */
class PaymentCompletePage extends ControllerBase {

  use TransactionTrait;

  /**
   * @var \Drupal\cg_payment\Manager\RequestManager
   */
  protected $cg_request_manager;

  /**
   * Constructs a PaymentCompletePage object
   */
  public function __construct($cg_request_manager) {
    $this->cg_request_manager = $cg_request_manager;
  }

  /**
   * @inheritdoc
   */
  public static function create(ContainerInterface $container) {
    $cg_request_manager = $container->get('cg_payment.cg_request_manager');
    return new static($cg_request_manager);
  }

  /**
   * Verify that the user has access to the payment complete page.
   *
   * @return \Drupal\Core\Access\AccessResultAllowed|\Drupal\Core\Access\AccessResultForbidden
   */
  public function access() {
    /** @var \Symfony\Component\HttpFoundation\Request $request */
    $request = \Drupal::request();
    $query_params = $request->query;

    $cg_redirect_return = new CgRedirectReturn($query_params);

    // Check if the query params are valid, otherwise throw 403
    $validation_errors = $cg_redirect_return->getValidationErrors();
    if (!empty($validation_errors)) {
      \Drupal::logger('cg_payment')
        ->notice('Invalid parameters received on CG payment complete page, possible hacking attempt. Messages: %messages', [
          '%messages' => implode(', ', $validation_errors)
        ]);
      return AccessResult::forbidden();
    }

    return AccessResult::allowed();
  }

  /**
   * The payment complete page where users finds themselves after payment.
   *
   * @return array
   */
  public function page(Request $request) {
    $query_params = $request->query;

    $cg_redirect_return = new CgRedirectReturn($query_params);

    $txnId = $cg_redirect_return->getTxId();
    $shva_code = $cg_redirect_return->getErrorCode();
    $transaction = $this->getTransactionByRemoteId($txnId, ['pending', 'success', 'failure']);

    // In case no entities found - return 404 (may be a fraud or an error)
    if (empty($transaction)) {
      throw new NotFoundHttpException();
    }

    // Check the loaded transaction status, if it is other than pending 0 that
    // means that the user probably refreshed the page.
    $transaction_status = $transaction->get('status')->value;
    if ($transaction_status != 'pending') {
      // Show the right page content according to the existing transaction
      return $this->getPageContent($request, $transaction, ($transaction_status == 'success'));
    }

    // Set the transaction data (without changing the status)
    $transaction->set('shva_code', $shva_code)
      ->set('last_digits', $cg_redirect_return->getLastDigits())
      ->set('expiration_date', $cg_redirect_return->getCardExp())
      ->save();

    if ($cg_redirect_return->isSuccessCode()) {
      // Log successful transactions to log
      \Drupal::logger('cg_payment')
        ->info('Token request completed successfully: TxID: @txnid, internal transaction ID: @tid', [
          '@txnid' => $txnId,
          '@tid' => $transaction->id()
        ]);

      // Make the actual charge request now that we have the token
      $charge_response = $this->cg_request_manager->requestChargeToken($txnId, $cg_redirect_return->getCardToken(), $cg_redirect_return->getCardExp());
    }
    else {
      // Failure - Error
      $error_text = $query_params->get('ErrorText');

      \Drupal::logger('cg_payment')
        ->warning('Token request failed: TxID: @txnid, Shva Code: @errorCode, Error Text (from CG): @errorText', [
          '@txnid' => $txnId,
          '@errorCode' => $shva_code,
          '@errorText' => $cg_redirect_return->getErrorText()
        ]);

      $transaction->set('status', 'failure')->save();
    }

    // Return the page's renderable array
    return $this->getPageContent($request, $transaction, !empty($charge_response));
  }

  /**
   * Method to return the page content
   * @param \Symfony\Component\HttpFoundation\Request $request
   * @param $status
   *
   * @return array
   *   Renderable array
   */
  protected function getPageContent(Request $request, TransactionInterface $transaction, $is_success) {
    $query_params = $request->query;

    return [
      '#theme' => 'payment_complete',
      '#attributes' => [],
      '#is_success' => $is_success,
      '#transaction' => $transaction,
      '#cg_redirect_return' => new CgRedirectReturn($query_params)
    ];
  }

}