<?php

namespace Drupal\cg_payment\Manager;

use Creditguard\CgCommandRequestChargeToken;
use Creditguard\CgCommandRequestPaymentFormUrl;
use Drupal\cg_payment\CgChargeEvent;
use Drupal\cg_payment\Entity\Transaction;
use Drupal\cg_payment\RequestInterface;
use Drupal\cg_payment\Utility\TransactionTrait;
use Drupal\Core\Config\ConfigFactory;
use Drupal\Core\TempStore\PrivateTempStoreFactory;
use Symfony\Component\EventDispatcher\EventDispatcherInterface;

/**
 * Class RequestManager
 * @package Drupal\cg_payment\Manager
 */
class RequestManager implements RequestInterface {

  use TransactionTrait;

  protected $config;
  protected $successUrl;
  protected $cancelUrl;
  protected $errorUrl;
  protected $transaction;

  /**
   * @var PrivateTempStoreFactory
   */
  protected $privateTempStore;

  protected $dispatcher;

  /**
   * @inheritdoc
   */
  public function __construct(ConfigFactory $config, PrivateTempStoreFactory $privateTempStore, EventDispatcherInterface $dispatcher) {
    $this->config = $config;
    $this->privateTempStore = $privateTempStore;
    $this->dispatcher = $dispatcher;

    $base_url = \Drupal::request()->getSchemeAndHttpHost();

    // Url to redirect user after successful payment
    $this->successUrl = "${base_url}/payment-complete";
    // Url to redirect user in case he cancel
    $this->cancelUrl = "${base_url}/payment-complete";
    // Url to redirect user in case of payment failure
    $this->errorUrl = "${base_url}/payment-complete";
  }

  /**
   * @inheritdoc
   * @todo Move some of the params to a different class.
   */
  public function requestChargeToken($txId, $token, $cardExp, $terminalNumber = NULL, $mid = NULL, $amount = NULL) {
    $transaction = $this->getTransactionByRemoteId($txId);

    // Quit if no transaction can be found with that txId
    if (empty($transaction)) {
      return FALSE;
    }

    // Get credentials from our configuration
    // @todo Move config to another service
    $cg_config = $this->config->get('cg_payment.settings');
    $relayUrl = $cg_config->get('endpoint_url');
    $username = $cg_config->get('user_name');
    $password = $cg_config->get('password');

    // Try getting the charge data from our session storage (saved on the
    // requestPaymentFormUrl method) in case the function haven't receive
    // those details.
    $tempstore = $this->privateTempStore->get('cg_payment');
    $terminalNumber = !empty($terminalNumber) ? $terminalNumber : $tempstore->get('terminal_number');
    $amount = !empty($amount) ? $amount : $tempstore->get('amount');
    $mid = !empty($mid) ? $mid : $tempstore->get('mid');

    if (empty($terminalNumber) || empty($mid) || empty($amount)) {
      // Log the bad call to watchdog
      \Drupal::logger('cg_payment')
        ->notice('Trying to charge token, but some data is missing: TxID: @txnid, terminal id: @terminal_id, mid: @mid, amount: @amount', [
          '@txnid' => $txId,
          '@terminal_id' => $terminalNumber ?: '',
          '@mid' => $mid ?: '',
          '@amount' => $amount ?: ''
        ]);

      return FALSE;
    }

    try {
      $request = new CgCommandRequestChargeToken($relayUrl, $username, $password, $terminalNumber, $mid);
      $request
        ->setTotal($amount)
        ->setCardToken($token)
        ->setCardExp($cardExp)
        ->setTxId($txId);

      // Dispatch event to allow other modules interact with the data before
      // making the actual request to CG (e.g. for adding extra data).
      $event = new CgChargeEvent($request, $transaction);
      $this->dispatcher->dispatch(CgChargeEvent::PRE_CHARGE, $event);

      /**
       * @var $charge_response CgCommandRequestChargeToken
       */
      $charge_response = $request
        ->execute();
      $is_success = $charge_response->isSuccessCode();

      $authNumber = !empty($charge_response->getResponse()->doDeal->authNumber) ? reset($charge_response->getResponse()->doDeal->authNumber) : '-1';
      $transaction->set('expiration_date', $cardExp)
        ->set('confirm_num', $authNumber)
        ->set('status', $is_success ? 'success' : 'failure')
        ->save();

      \Drupal::logger('cg_payment')
        ->info('CG charge token request completed: TxID: @txnid, internal transaction ID: @tid', [
          '@txnid' => $txId,
          '@tid' => $transaction->id()
        ]);

      return $is_success ? $transaction : FALSE;
    }
    catch (\Exception $e) {
      \Drupal::logger('cg_payment')
        ->error('CG charge token request failed: TxID: @txnid, error code: @code, error message: @message', [
          '@txnid' => $txId,
          '@code' => $e->getCode(),
          '@message' => $e->getMessage()
        ]);
    }

    return FALSE;
  }

  /**
   * @inheritdoc
   */
  public function requestPaymentFormUrl($terminalNumber, $mid, $amount, $email, $description) {
    $url = NULL;

    $this->transaction = Transaction::create();

    // Get credentials from our configuration
    $cg_config = $this->config->get('cg_payment.settings');
    $relayUrl = $cg_config->get('endpoint_url');
    $username = $cg_config->get('user_name');
    $password = $cg_config->get('password');

    try {
      $request = new CgCommandRequestPaymentFormUrl($relayUrl, $username, $password, $terminalNumber, $mid);
      $request
        ->setTotal($amount)
        ->setEmail($email)
        ->setDescription($description)
        ->setSuccessUrl($this->successUrl)
        ->setCancelUrl($this->cancelUrl)
        ->setErrorUrl($this->errorUrl);

      $cg_response = $request->execute();

      $redirect_url = $cg_response->getPaymentFormUrl();
      $token_id = $cg_response->getPaymentFormToken();

      $this->transaction
        ->set('status', 'pending')
        ->set('remote_id', $token_id);

      $url = $redirect_url;
    }
    catch (\Exception $e) {
      $this->transaction
        ->set('status', 'failure');
      \Drupal::messenger()->addError(
        t('An error as occurred, please try again.')
      );
      print $e->getMessage();
    }

    $this->transaction->save();

    // Save data for later usage (e.g. the actual token charge)
    $tempstore = $this->privateTempStore->get('cg_payment');
    $tempstore->set('amount', $amount);
    $tempstore->set('terminal_number', $terminalNumber);
    $tempstore->set('mid', $mid);

    return $url;
  }

  /**
   * @param mixed $successUrl
   * @return RequestManager
   */
  public function setSuccessUrl($successUrl) {
    $this->successUrl = $successUrl;
    return $this;
  }

  /**
   * @param mixed $cancelUrl
   * @return RequestManager
   */
  public function setCancelUrl($cancelUrl) {
    $this->cancelUrl = $cancelUrl;
    return $this;
  }

  /**
   * @param mixed $errorUrl
   * @return RequestManager
   */
  public function setErrorUrl($errorUrl) {
    $this->errorUrl = $errorUrl;
    return $this;
  }

  /**
   * @return mixed
   */
  public function getTransaction() {
    return $this->transaction;
  }

}
